package org.mentalog.test;

import static org.mentalog.Log.Warn;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;
import org.mentalog.Log;
import org.mentalog.util.Benchmarker;
import org.slf4j.LoggerFactory;

public class PerformanceComparator {
	
	private enum LogLibrary { LOG4J, MENTALOG, LOGBACK };

	private static final String MSG = "This is a log message not so small that you can use to test. I hope you have fun and it works well!";
	
	private static final String FORMAT = "format HeapTest <max> <mentalog|log4j|logback>";

	public static void main(String[] args) {

		// java -cp target/classes:lib/mentalog-license.jar:lib/slf4j-api-1.6.2.jar:lib/logback-core-0.9.30.jar:lib/logback-classic-0.9.30.jar:lib/log4j-1.2.16.jar -Dlogback.configurationFile=./src/main/java/org/mentalog/test/logback.xml org.mentalog.test.PerformanceComparator 100000 mentalog

		if (args.length != 2) {
			System.out.println(FORMAT);
			return;
		}
		
		LogLibrary logLibrary = null;
		
		try {
			logLibrary = Enum.valueOf(LogLibrary.class, args[1].toUpperCase());
		} catch(Exception e) {
			System.err.println("Cannot parse log library: " + args[1]);
			System.out.println(FORMAT);
			return;
		}

		int max = Integer.parseInt(args[0]);

		long totalMemory = 0;
		long logsWithNoMemory = 0;
		long logsWithMemory = 0;
		long logsTotal = 0;
		long logsGC = 0;
		
		Benchmarker bench = new Benchmarker();
		
		setup(logLibrary);
		
		for (int i = 1; i <= max; i++) {
			long freeMemory = Runtime.getRuntime().freeMemory();
			bench.mark();
			log(logLibrary);
			bench.measure();
			long mem = freeMemory - Runtime.getRuntime().freeMemory();
			logsTotal++;
			if (mem > 0) {
				//System.out.println("Created " + mem + " bytes after " + i + " log calls");
				totalMemory += mem;
				logsWithMemory++;
			} else if (mem == 0) {
				logsWithNoMemory++;
			} else {
				//System.out.println("GC called!");
				logsGC++;
			}
		}
		
		System.out.println("Wrote " + logsTotal + " log lines to a file!");
		System.out.println("Logs that allocated memory: " + logsWithMemory + " (" + p(logsWithMemory, logsTotal) + ")");
		System.out.println("Logs that did not allocate any memory: " + logsWithNoMemory + " (" + p(logsWithNoMemory, logsTotal) + ")");
		System.out.println("Logs that triggered GC: " + logsGC + " (" + p(logsGC, logsTotal) + ")");
		System.out.println("Memory allocated: " + totalMemory + " bytes");
		System.out.println("Benchmark: " + bench.results());
		System.out.println();
	}
	
	private static org.slf4j.Logger logger_logback = null;
	private static org.apache.log4j.Logger logger_log4j = null;
	
	private final static void setup(LogLibrary logLibrary) {
		if (logLibrary == LogLibrary.MENTALOG) {
			Log.setFile(true);
			Warn.log(MSG); // force the log to open to create the file and get ready to log (that of course creates memory!)
		} else if (logLibrary == LogLibrary.LOGBACK) {
			logger_logback = LoggerFactory.getLogger("chapters.introduction.HelloWorld1");
			logger_logback.warn(MSG);
		} else if (logLibrary == LogLibrary.LOG4J) {
			PropertyConfigurator.configure("./src/main/java/org/mentalog/test/log4j.properties");
			logger_log4j = Logger.getLogger(GCTest.class);
			logger_log4j.warn(MSG);
		}
	}
	
	private final static void log(LogLibrary logLibrary) {
		if (logLibrary == LogLibrary.MENTALOG) {
			Warn.log(MSG);
		} else if (logLibrary == LogLibrary.LOGBACK) {
			logger_logback.warn(MSG);
		} else if (logLibrary == LogLibrary.LOG4J) {
			logger_log4j.warn(MSG);
		}
	}
	
	private static String p(long l, long total) {
		double d = ((double) l) / ((double) total) * 100D;
		double rounded = Math.round(d * 100) / 100D;
		return String.valueOf(rounded) + "%";
	}
}